;********************************************************
; FICHIER........: DifAilerons.ASM
; AUTEUR ........: S. Lichtenberger
; DESCRIPTION ...: Module pour diffrentiel d'ailerons
; NOTES .........:
; COPYRIGHT .....: S. Lichtenberger
; Version .......: 14/11/2004
;********************************************************

PROCESSOR	PIC12F675
INCLUDE	<P12F675.inc>
__config  _MCLRE_OFF & _CP_OFF  & _INTRC_OSC_NOCLKOUT & _WDT_OFF

; Dfinition  GPIO et TRIS
           ; 
OutA1 EQU 4 ; Aileron 1
OutA2 EQU 5 ; Aileron 2
InPulse EQU	3 ; Entre Pulse
Fonction EQU 0 ; en attente

TrisBits        EQU  B'001111'

OptionBits	EQU	B'00000000' ; Pull up ON

#DEFINE Page0 Bcf STATUS,5
#DEFINE Page1 Bsf STATUS,5

; 1500=0x5DC Pour repos servo  1500 s
#DEFINE ServoReposH 5
#DEFINE ServoReposL 0xDC 

; 1600=0x640 Pour repos servo  1500 s
;#DEFINE ServoReposH 6
;#DEFINE ServoReposL 0x40

org 0x20
ValeurPotar RES 1
ValeurPulseH RES 1
ValeurPulseL RES 1
RH RES 1
RL RES 1
PulseOutH RES 1
PulseOutL RES 1
ValTH RES 1
ValTL RES 1
Compteur1 RES 1


;================================================
;Tempo de PulseOutH, PulseOutL 
; Utiliser le timer 0 pour les tempo des pulse de sortie

TempoPOut macro OUT
local AttInt

bcf T1CON,TMR1ON ; arret timer 
bcf PIR1,TMR1IF
clrf ValTL
movfw pulseoutl
subwf ValTL,W
btfss STATUS,C
incf PulseOutH
movwf TMR1L
clrf ValTH
movfw pulseouth
subwf ValTH,W
movwf TMR1H
bsf T1CON,TMR1ON ; timer ON
bsf GPIO,OUT

AttInt:
btfss PIR1,TMR1IF
goto AttInt
bcf GPIO,OUT

endm


; =========================================================================
; Dpart

Coldstart:	ORG	0

page1
call 3ffh
movwf OSCCAL		; Init Oscal pour 4 MHz
movlw OptionBits
movwf OPTION_REG
movlw Trisbits
movwf TRISIO
movlw b'01010100'
movwf ANSEL ; AN3 select en analogique
page0
movlw b'00001001'
movwf ADCON0 ; adresse H, vfref = VDD, Chanel 2, convertisseur ON
movlw b'00000001' ; Validation timer 1
movwf T1CON
clrf	GPIO			; Prog Sortie  0

; Fin des initialisations

Main:
call LecturePotar
call lecturePulse
call pulsesOUT

goto Main

LecturePulse:
btfsc GPIO,InPulse
goto LecturePulse ; attente entre basse

AtPulseH:
btfss GPIO,InPulse
goto AtPulseH ; attente entre haute
clrf TMR1L
clrf TMR1H

AtFinPulse:
btfsc GPIO,InPulse
goto AtFinPulse ; attente entre basse
movfw TMR1H
movwf ValeurPulseH
movfw TMR1L
movwf ValeurPulseL
movfw TMR1H
subwf ValeurPulseH,W
btfsc STATUS,Z
goto LectureOK
movfw TMR1H ; On a eu un roll over
movwf ValeurPulseH
movfw TMR1L
movwf ValeurPulseL

LectureOK:
return ; le pulse vaut ValeurPulse (H L) s

;================================================
lecturePotar:
bsf ADCON0,GO_DONE ; dpart conversion

BclAtADC:
btfsc ADCON0,GO_DONE ; Attente fin de conversion
goto BclAtADC

movfw ADRESH
movwf ValeurPotar
rrf ValeurPotar

rrf ValeurPotar ; -> division par 8
movlw b'00111111'
andwf ValeurPotar ; Valeur comprise entre 0 et 64
btfsc STATUS,Z
incf ValeurPotar ; Elimination valeur 0
return

;================================================
;Rev = 0FF
; Si Pulse IN > 1500s, PulseOUT2=PulseIN
; et PulseOUT1 = 1500+(PulseIN-1500)n/64
; avec n compris entre 1 et 64 selon le potar

; Si Pulse IN < 1500s, PulsOUT1=PulseIN
; et PulseOUT2 = 1500-(1500-PulseIN)n/64
; avec n compris entre 1 et 64 selon le potar
;================================================
;Rev = ON
; Si Pulse IN > 1500s, PulseOUT1=PulseIN
; et PulseOUT2 = 1500+(PulseIN-1500)n/64
; avec n compris entre 1 et 64 selon le potar

; Si Pulse IN < 1500s, PulsOUT2=PulseIN
; et PulseOUT1 = 1500-(1500-PulseIN)n/64
; avec n compris entre 1 et 64 selon le potar
;================================================


PulsesOUT:
; Le pulse mdian pour ValeurPulse 1500 = 0x5DC
movfw ValeurPulseH
sublw ServoReposH
btfsc STATUS,Z
goto TestValeurPulseL
movfw ValeurPulseH
sublw ServoReposH
btfsc STATUS,C
goto PulseInf1500
goto PulseSup1500

TestValeurPulseL:
movfw ValeurPulseL
sublw ServoReposL
btfsc STATUS,C
goto PulseInf1500

;================================================
PulseSup1500: ; Pulse > 1500 s
;Pulse IN > 1500s, PulseOUT2=PulseIN

movfw ValeurPulseH
movwf PulseOutH
movfw ValeurPulseL
movwf PulseOutL
TempoPOut OutA2 ; Fin du pulse A2

; Calcul PulseOUT1 = 1500+(PulseIN-1500)n/64
movlw ServoReposL
subwf ValeurPulseL,W
movwf PulseOutL ; Rsultat de la soustraction
btfsc STATUS,C ; retenue ?
goto PasDeRetenue
movlw ServoReposH + 1
subwf ValeurPulseH,W
goto S1
 
PasDeRetenue:
movlw ServoReposH
subwf ValeurPulseH,W

S1:
movwf PulseOutH
clrf RH  ; PulseIN-1500 c'est fait, le multiplier par n=valeur potar
clrf RL
movfw ValeurPotar
movwf Compteur1
; Ajouter n fois PulseOut  
movfw PulseOutL

Bcl2:
addwf RL
btfsc STATUS,C ; retenue ?
incf RH
decfsz Compteur1
goto Bcl2
movfw ValeurPotar
movwf Compteur1
movfw PulseOutH

Bcl3:
addwf RH
decfsz Compteur1
goto Bcl3
; Multiplication faite dans RH RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL

; On a divis par 64
; Reste  ajouter 1500 (0xDC)
movlw ServoReposL
addwf RL
btfsc STATUS,C
incf RH
movlw ServoReposH
addwf RH

; RH et RL contielle la valeur du pulse corrig, on le sort
movfw RH
movwf PulseOutH
movfw RL
movwf PulseOutL
TempoPOut OutA1 ; Fin du pulse A1
return

;================================================
PulseInf1500: ; Pulse < 1500 s
;Pulse IN > 1500s, PulseOUT1=PulseIN

movfw ValeurPulseH
movwf PulseOutH
movfw ValeurPulseL
movwf PulseOutL
TempoPOut OutA1 ; Fin du pulse A1
movfw ValeurPulseL ; On va calculer PulseOUT1 = 1500-(1500-PulseIN)n/64
sublw ServoReposL
movwf PulseOutL
btfsc STATUS,C
goto PasDeRetenue1
movfw ValeurPulseH
addlw 1

Bcl6:
sublw ServoReposH
movwf PulseOutH
goto Suite1

PasDeRetenue1:
movfw ValeurPulseH
goto Bcl6

Suite1: ; Multiplier PulseOutH PulseOutL par n
clrf RH 
clrf RL
movfw ValeurPotar
movwf Compteur1
; Ajouter n fois PulseOut  
movfw PulseOutL

Bcl2i:
addwf RL
btfsc STATUS,C ; retenue ?
incf RH
decfsz Compteur1
goto Bcl2i
movfw ValeurPotar
movwf Compteur1
movfw PulseOutH

Bcl3i:
addwf RH
decfsz Compteur1
goto Bcl3i

; Multiplication faite dans RH RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL
bcf STATUS,C
rrf RH
rrf RL

; On a divis par 64
; Reste  faire 1500 - (RH RL)
movfw RL
sublw ServoReposL
movwf PulseOutL
btfsc STATUS,C
goto PasDeRetenue1i
movfw RH
addlw 1

Bcl6i:
sublw ServoReposH
movwf PulseOutH
goto Suite1i

PasDeRetenue1i:
movfw RH
goto Bcl6i

; Le rsultat est dans PulseOut, sortir le pulse 2
Suite1i:

TempoPOut OutA2 ; Fin du pulse A2
return


;================================================
;ORG 0x3ff
;db 0x35,0xA0 ; Valeur OSCAL - A lire pour chaque Proc avant son effacement.
END

